import { CommonModule } from '@angular/common';
import { TextMaskModule } from 'angular2-text-mask';
import { TryCatchMaterialExtensionModule } from '@trycatch/material-extension';
import '@angular/cdk/text-field';
import { take } from 'rxjs/operators';
import createAutoCorrectedDatePipe from 'text-mask-addons/dist/createAutoCorrectedDatePipe';
import { MAT_DATE_LOCALE, DateAdapter, MAT_DATE_FORMATS, MatCommonModule, MatFormFieldModule, MatInputModule, MatDatepickerModule, MatSelectModule, MatProgressSpinnerModule } from '@angular/material';
import { MomentDateAdapter, MatMomentDateModule } from '@angular/material-moment-adapter';
import * as moment_ from 'moment';
import { Input, Output, EventEmitter, Component, Optional, Inject, NgZone, ViewChild, NgModule } from '@angular/core';
import { FormGroupDirective, FormControl, FormsModule, ReactiveFormsModule } from '@angular/forms';
import 'rxjs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const FormFieldControlType = {
    INPUT_TEXT: "InputText",
    INPUT_NUMBER: "InputNumber",
    AUTOCOMPLETE: "AutoComplete",
};
/** @enum {string} */
const FormFieldEventType = {
    INPUT_CHANGED: "input_changed",
    OPTION_SELECTED: "option_selected",
    CONTROL_CREATED: "control_created",
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} event
 * @return {?}
 */
function numericKeysOnly(event) {
    /** @type {?} */
    const charCode = event.which ? event.which : event.keyCode;
    if (charCode >= 48 && charCode <= 57) {
        return true;
    }
    return false;
}
/**
 * @param {?} event
 * @return {?}
 */
function numericKeysOnlyWithDecimalPoint(event) {
    /** @type {?} */
    const charCode = event.which ? event.which : event.keyCode;
    if ((charCode >= 48 && charCode <= 57) || charCode === 46) {
        return true;
    }
    return false;
}
/**
 * @param {?} event
 * @return {?}
 */
function alphaNumericKeysOnly(event) {
    /** @type {?} */
    const charCode = event.which ? event.which : event.keyCode;
    if (charCode === 32 ||
        (charCode >= 97 && charCode <= 122) ||
        (charCode >= 65 && charCode <= 90) ||
        (charCode >= 48 && charCode <= 57) // numeric keys
    ) {
        return true;
    }
    return false;
}
/**
 * @param {?} event
 * @return {?}
 */
function alphaKeysOnly(event) {
    /** @type {?} */
    const charCode = event.which ? event.which : event.keyCode;
    if (charCode >= 48 && charCode <= 57) {
        // numeric keys
        return false;
    }
    if (charCode >= 33 && charCode <= 38) {
        // special keys
        return false;
    }
    if (charCode >= 40 && charCode <= 44) {
        // special keys
        return false;
    }
    if (charCode >= 46 && charCode <= 47) {
        // special keys
        return false;
    }
    if (charCode >= 58 && charCode <= 64) {
        // special keys
        return false;
    }
    if (charCode >= 91 && charCode <= 95) {
        // special keys
        return false;
    }
    if (charCode >= 123 && charCode <= 126) {
        // special keys
        return false;
    }
    if (charCode >= 161 && charCode <= 191) {
        // special keys
        return false;
    }
    if (charCode === 8217) {
        // special keys
        return false;
    }
    return true;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class FromFieldControl {
    /**
     * @param {?} __formGroupDirective
     */
    constructor(__formGroupDirective) {
        this.keyRestrictionsFunctionMap = {
            "numeric-only": numericKeysOnly,
            "numeric-with-decimal-point": numericKeysOnlyWithDecimalPoint,
            "alpha-numeric": alphaNumericKeysOnly,
            "alpha-only": alphaKeysOnly
        };
        this.event = new EventEmitter();
        this._keyRestrictionFunction = this.__keyRestrictionFunction;
        this.formGroupDirective = __formGroupDirective;
        this.event.emit((/** @type {?} */ ({
            type: FormFieldEventType.CONTROL_CREATED
        })));
        if (!__formGroupDirective) {
            console.error("form field controls can be used within form group container");
        }
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set keyRestrictions(val) {
        if (!!val) {
            this._keyRestrictionFunction = this.keyRestrictionsFunctionMap[val];
        }
        else {
            this._keyRestrictionFunction = this.__keyRestrictionFunction;
        }
    }
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    __keyRestrictionFunction(event) {
        return true;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if ("controlName" in changes) {
            if (this.formGroupDirective) {
                this.formGroup = this.formGroupDirective.form;
                this._control = this.formGroup.controls[changes.controlName.currentValue];
                this.__setValidators();
                this.__pendingSubscription();
            }
        }
    }
    /**
     * @return {?}
     */
    _runReplacement() {
        if (this.replacementMap &&
            this.replacementMap.length > 0 &&
            this._control) {
            /** @type {?} */
            let val = this._control.value.toString();
            if (val) {
                this.replacementMap.forEach((/**
                 * @param {?} rm
                 * @return {?}
                 */
                rm => {
                    /** @type {?} */
                    const regEx = new RegExp(rm[0].source, rm[0].flags + "g");
                    val = val.replace(regEx, rm[1]);
                }));
                this._control.setValue(val);
            }
        }
    }
    /**
     * @private
     * @return {?}
     */
    __setValidators() {
        if (this.validationFn) {
            this._control.setValidators(this.validationFn);
        }
    }
    /**
     * @private
     * @return {?}
     */
    __pendingSubscription() {
        if (this._control) {
            this._control.statusChanges.subscribe((/**
             * @param {?} status
             * @return {?}
             */
            (status) => {
                /** @type {?} */
                const msg = `${this.controlName} status is : ${status}`;
                console.log(msg);
                console.log(this._control.errors);
                this._status = status;
            }));
        }
    }
    /**
     * @protected
     * @return {?}
     */
    get __control() {
        return this._control;
    }
    /**
     * @param {?} controlType
     * @return {?}
     */
    __controlInfo(controlType) {
        return {
            name: this.controlName,
            type: controlType,
            abstractControl: this.__control
        };
    }
    /**
     * @return {?}
     */
    get valid() {
        return this.__control
            ? this.__control.dirty &&
                !this.__control.errors &&
                !this.__control.pending
            : undefined;
    }
    /**
     * @return {?}
     */
    get error() {
        if (this.__control) {
            /** @type {?} */
            const ctrl = this.__control;
            if (ctrl.errors) {
                if (ctrl.hasError("required")) {
                    return `${this.label} is required`;
                }
                if (ctrl.hasError("error")) {
                    return `${ctrl.errors["error"]}`;
                }
                if (this.errorFn) {
                    /** @type {?} */
                    const err = this.errorFn(ctrl.errors, this.parent);
                    if (err) {
                        return err;
                    }
                    else {
                        setTimeout((/**
                         * @return {?}
                         */
                        () => {
                            ctrl.setErrors(null);
                        }));
                    }
                }
            }
        }
        return undefined;
    }
    /**
     * @protected
     * @param {?} val
     * @return {?}
     */
    _cleanMaskValue(val) {
        return val ? val.replace(/[^\x00-\x7F]/g, "") : val;
    }
}
FromFieldControl.propDecorators = {
    label: [{ type: Input }],
    placeholder: [{ type: Input }],
    hint: [{ type: Input }],
    controlName: [{ type: Input }],
    required: [{ type: Input }],
    validImage: [{ type: Input }],
    appearance: [{ type: Input }],
    floatLabel: [{ type: Input }],
    color: [{ type: Input }],
    invalidImage: [{ type: Input }],
    parent: [{ type: Input }],
    validationFn: [{ type: Input }],
    errorFn: [{ type: Input }],
    keyRestrictions: [{ type: Input }],
    replacementMap: [{ type: Input }],
    event: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InputFormFieldComponent extends FromFieldControl {
    /**
     * @param {?} __formGroupDirective
     */
    constructor(__formGroupDirective) {
        super(__formGroupDirective);
        this.maxLength = -1;
        this.minLength = -1;
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
InputFormFieldComponent.decorators = [
    { type: Component, args: [{
                selector: "tc-input-form-field",
                template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<input\n\t\tmatInput\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t(keypress)=\"_keyRestrictionFunction($event)\"\n\t\t(blur)=\"_runReplacement()\"\n\t\t[required]=\"required\"\n\t\t[maxlength]=\"maxLength > 0 ? maxLength : undefined\"\n\t\ttype=\"text\"\n\t/>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<mat-spinner\n\t\t*ngIf=\"_status === 'PENDING'\"\n\t\t[diameter]=\"18\"\n\t\t[strokeWidth]=\"3\"\n\t></mat-spinner>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
            }] }
];
/** @nocollapse */
InputFormFieldComponent.ctorParameters = () => [
    { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
];
InputFormFieldComponent.propDecorators = {
    maxLength: [{ type: Input }],
    minLength: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AutoCompleteFormFieldComponent extends FromFieldControl {
    /**
     * @param {?} __formGroupDirective
     */
    constructor(__formGroupDirective) {
        super(__formGroupDirective);
        this.charThreshold = 2;
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
    /**
     * @param {?} eventData
     * @return {?}
     */
    _optionSelected(eventData) {
        /** @type {?} */
        const eventVal = (/** @type {?} */ ({
            type: FormFieldEventType.OPTION_SELECTED,
            control: this.__controlInfo(FormFieldControlType.AUTOCOMPLETE)
        }));
        if (eventData) {
            eventVal.data = eventData;
        }
        else {
            this.__control.reset();
        }
        this.event.emit(eventVal);
    }
    /**
     * @param {?} eventData
     * @return {?}
     */
    _inputChanged(eventData) {
        this.event.emit((/** @type {?} */ ({
            type: FormFieldEventType.INPUT_CHANGED,
            data: eventData,
            control: this.__controlInfo(FormFieldControlType.AUTOCOMPLETE)
        })));
    }
}
AutoCompleteFormFieldComponent.decorators = [
    { type: Component, args: [{
                selector: "tc-auto-complete-form-field",
                template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<trycatch-mat-auto-complete\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[options]=\"options\"\n\t\t[displayProperty]=\"displayProperty\"\n\t\t[displayFn]=\"displayFn\"\n\t\t[charThreshold]=\"charThreshold\"\n\t\t[optionDisplayFn]=\"optionDisplayFn\"\n\t\t[optionClassFn]=\"optionClassFn\"\n\t\t[required]=\"required\"\n\t\t[options$]=\"options$\"\n\t\t(optionSelected)=\"_optionSelected($event)\"\n\t\t(inputChanged)=\"_inputChanged($event)\"\n\t\t[classList]=\"classList\"\n\t>\n\t</trycatch-mat-auto-complete>\n\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
            }] }
];
/** @nocollapse */
AutoCompleteFormFieldComponent.ctorParameters = () => [
    { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
];
AutoCompleteFormFieldComponent.propDecorators = {
    options: [{ type: Input }],
    options$: [{ type: Input }],
    displayProperty: [{ type: Input }],
    displayFn: [{ type: Input }],
    optionDisplayFn: [{ type: Input }],
    optionClassFn: [{ type: Input }],
    charThreshold: [{ type: Input }],
    classList: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NumberFormFieldComponent extends FromFieldControl {
    /**
     * @param {?} __formGroupDirective
     */
    constructor(__formGroupDirective) {
        super(__formGroupDirective);
        this.step = 0.1;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this._control.valueChanges.subscribe((/**
         * @param {?} val
         * @return {?}
         */
        val => {
            if (val) {
                if (this.minValue) {
                    if (val < this.minValue) {
                        console.log("setting up min value error");
                        this._control.setErrors({
                            error: "value out of range. Value less then min allowed value"
                        });
                    }
                }
                if (this.maxValue) {
                    if (val > this.maxValue) {
                        console.log("setting up max value error");
                        this._control.setErrors({
                            error: "value out of range. Value greater then max allowed value"
                        });
                    }
                }
            }
            else {
                this._control.setErrors(null);
            }
        }));
    }
}
NumberFormFieldComponent.decorators = [
    { type: Component, args: [{
                selector: "tc-number-form-field",
                template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<input\n\t\tmatInput\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[required]=\"required\"\n\t\ttype=\"number\"\n\t\t[step]=\"step\"\n\t\t[attr.min]=\"minValue\"\n\t\t[attr.max]=\"maxValue\"\n\t/>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
            }] }
];
/** @nocollapse */
NumberFormFieldComponent.ctorParameters = () => [
    { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
];
NumberFormFieldComponent.propDecorators = {
    step: [{ type: Input }],
    minValue: [{ type: Input }],
    maxValue: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TextAreaFormFieldComponent extends FromFieldControl {
    /**
     * @param {?} ngZone
     * @param {?} __formGroupDirective
     */
    constructor(ngZone, __formGroupDirective) {
        super(__formGroupDirective);
        this.ngZone = ngZone;
        this.minRows = 3;
        this.maxRows = 15;
    }
    /**
     * @return {?}
     */
    triggerResize() {
        // Wait for changes to be applied, then trigger textarea resize.
        this.ngZone.onStable
            .pipe(take(1))
            .subscribe((/**
         * @return {?}
         */
        () => this.autosize.resizeToFitContent(true)));
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
TextAreaFormFieldComponent.decorators = [
    { type: Component, args: [{
                selector: "tc-text-area-form-field",
                template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<textarea\n\t\tmatInput\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[required]=\"required\"\n\t\ttype=\"text\"\n\t\tcdkTextareaAutosize\n\t\t#autosize=\"cdkTextareaAutosize\"\n\t\t[cdkAutosizeMinRows]=\"minRows\"\n\t\t[cdkAutosizeMaxRows]=\"maxRows\"\n\t></textarea>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
            }] }
];
/** @nocollapse */
TextAreaFormFieldComponent.ctorParameters = () => [
    { type: NgZone },
    { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
];
TextAreaFormFieldComponent.propDecorators = {
    minRows: [{ type: Input }],
    maxRows: [{ type: Input }],
    cols: [{ type: Input }],
    autosize: [{ type: ViewChild, args: ["autosize",] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MaskedFormFieldComponent extends FromFieldControl {
    /**
     * @param {?} __formGroupDirective
     */
    constructor(__formGroupDirective) {
        super(__formGroupDirective);
        this._placeholderDate = "01";
        this._placeholderMonth = "01";
        this._placeholderYear = "1900";
        this.maxLength = -1;
        this.minLength = -1;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set mask(val) {
        if (val) {
            this._mask = {
                mask: val,
                guide: true,
                placeholderChar: "\u2000"
            };
            if (this._autoCorrectedDatePipe) {
                this._mask.pipe = this._autoCorrectedDatePipe;
            }
        }
    }
    /**
     * @param {?} format
     * @return {?}
     */
    set dateFormat(format) {
        if (format) {
            this._autoCorrectedDatePipe = createAutoCorrectedDatePipe(format);
            this.placeholder = format
                .replace("dd", this._placeholderDate)
                .replace("mm", this._placeholderMonth)
                .replace("yyyy", this._placeholderYear);
            if (this._mask) {
                this._mask.pipe = this._autoCorrectedDatePipe;
            }
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
MaskedFormFieldComponent.decorators = [
    { type: Component, args: [{
                selector: "tc-masked-form-field",
                template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<input\n\t\tmatInput\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[required]=\"required\"\n\t\t[maxlength]=\"maxLength > 0 ? maxLength : undefined\"\n\t\t(keypress)=\"_keyRestrictionFunction($event)\"\n\t\ttype=\"text\"\n\t\t[textMask]=\"_mask\"\n\t/>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<mat-spinner\n\t\t*ngIf=\"_status === 'PENDING'\"\n\t\t[diameter]=\"18\"\n\t\t[strokeWidth]=\"3\"\n\t></mat-spinner>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
            }] }
];
/** @nocollapse */
MaskedFormFieldComponent.ctorParameters = () => [
    { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
];
MaskedFormFieldComponent.propDecorators = {
    maxLength: [{ type: Input }],
    minLength: [{ type: Input }],
    mask: [{ type: Input }],
    dateFormat: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DATE_FORMATS = {
    parse: {
        dateInput: "L"
    },
    display: {
        dateInput: "L",
        monthYearLabel: "MMM YYYY",
        dateA11yLabel: "L",
        monthYearA11yLabel: "MMMM YYYY"
    }
};
/** @type {?} */
const moment = moment_;
class MomentFormFieldComponent extends FromFieldControl {
    /**
     * @param {?} __formGroupDirective
     */
    constructor(__formGroupDirective) {
        super(__formGroupDirective);
        this.__dateMask = [
            /\d/,
            /\d/,
            "/",
            /\d/,
            /\d/,
            "/",
            /[1-2]/,
            /\d/,
            /\d/,
            /\d/
        ];
        this.__dpControl = new FormControl();
        this._mask = {
            mask: this.__dateMask,
            guide: true,
            placeholderChar: "\u2000",
            pipe: createAutoCorrectedDatePipe("dd/mm/yyyy")
        };
        this.placeholder = "01/01/1900";
        this.hint = "format: 01/01/1900";
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set startDate(val) {
        /** @type {?} */
        const mmt = moment(val, "DD/MM/YYYY");
        if (mmt.isValid()) {
            // this.__dpControl.setValue(mmt, { emitEvent: false });
            this._startDate = mmt;
            this._bkpStartDate = mmt;
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.__dpControl.valueChanges.subscribe((/**
         * @param {?} val
         * @return {?}
         */
        (val) => {
            /** @type {?} */
            const dt = val.format("DD/MM/YYYY");
            this._control.setValue(dt, { emitEvent: false });
            this._control.markAsTouched();
            this._startDate = val;
        }));
        this._control.valueChanges.subscribe((/**
         * @param {?} val
         * @return {?}
         */
        (val) => {
            /** @type {?} */
            const dt = this._cleanMaskValue(val);
            if (dt.length === 10) {
                /** @type {?} */
                const mmt = moment(dt, "DD/MM/YYYY");
                if (mmt.isValid()) {
                    this.__dpControl.setValue(mmt, { emitEvent: false });
                    this._startDate = mmt;
                }
            }
            else if (dt.length === 0) {
                this._startDate = this._bkpStartDate;
            }
        }));
    }
}
MomentFormFieldComponent.decorators = [
    { type: Component, args: [{
                selector: "tc-moment-form-field",
                template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<input\n\t\tmatInput\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[required]=\"required\"\n\t\t[textMask]=\"_mask\"\n\t/>\n\t<input\n\t\tmatInput\n\t\t[hidden]=\"true\"\n\t\t[formControl]=\"__dpControl\"\n\t\t[matDatepicker]=\"picker\"\n\t\t[required]=\"required\"\n\t\t[matDatepickerFilter]=\"dateFilterFn\"\n\t/>\n\t<mat-datepicker-toggle matSuffix [for]=\"picker\"></mat-datepicker-toggle>\n\t<mat-datepicker\n\t\t#picker\n\t\tdisabled=\"false\"\n\t\tstartView=\"month\"\n\t\t[startAt]=\"_startDate\"\n\t></mat-datepicker>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                providers: [
                    // `MomentDateAdapter` and `MAT_MOMENT_DATE_FORMATS` can be automatically provided by importing
                    // `MatMomentDateModule` in your applications root module. We provide it at the component level
                    // here, due to limitations of our example generation script.
                    { provide: MAT_DATE_LOCALE, useValue: "en-AU" },
                    {
                        provide: DateAdapter,
                        useClass: MomentDateAdapter,
                        deps: [MAT_DATE_LOCALE]
                    },
                    { provide: MAT_DATE_FORMATS, useValue: DATE_FORMATS }
                ],
                styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}:host mat-datepicker-toggle ::ng-deep button{width:30px;height:30px}:host mat-datepicker-toggle ::ng-deep button span.mat-button-wrapper{display:block}"]
            }] }
];
/** @nocollapse */
MomentFormFieldComponent.ctorParameters = () => [
    { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
];
MomentFormFieldComponent.propDecorators = {
    dateFilterFn: [{ type: Input }],
    startDate: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const MONTH_YEAR_FORMAT = {
    parse: {
        dateInput: "MM/YYYY"
    },
    display: {
        dateInput: "MM/YYYY",
        monthYearLabel: "MMM YYYY",
        dateA11yLabel: "LL",
        monthYearA11yLabel: "MMMM YYYY"
    }
};
/** @type {?} */
const moment$1 = moment_;
class ExpiryMonthFormFieldComponent extends FromFieldControl {
    /**
     * @param {?} __formGroupDirective
     */
    constructor(__formGroupDirective) {
        super(__formGroupDirective);
        this.__val = moment$1();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this._control.value) {
            this.__val = moment$1(this._control.value, "MM/YYYY");
            // } else {
            // 	this._control.setValue(this.__val);
        }
    }
    /**
     * @param {?} normalizedYear
     * @return {?}
     */
    _chosenYearHandler(normalizedYear) {
        console.log("control : ", this._control);
        /** @type {?} */
        const ctrlValue = this._control.value || this.__val;
        console.log("control value : ", ctrlValue);
        ctrlValue.year(normalizedYear.year());
        if (this._control.value) {
            this._control.setValue(ctrlValue);
        }
        else {
            this.__val = ctrlValue;
        }
        console.log("control value : ", this._control.value);
    }
    /**
     * @param {?} normlizedMonth
     * @param {?} datepicker
     * @return {?}
     */
    _chosenMonthHandler(normlizedMonth, datepicker) {
        /** @type {?} */
        const ctrlValue = this._control.value || this.__val;
        ctrlValue.month(normlizedMonth.month());
        if (this._control.value) {
            this._control.setValue(ctrlValue);
        }
        else {
            this.__val = ctrlValue;
            this._control.setValue(this.__val);
        }
        datepicker.close();
    }
}
ExpiryMonthFormFieldComponent.decorators = [
    { type: Component, args: [{
                selector: "tc-expiry-month-form-field",
                template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<input\n\t\tmatInput\n\t\treadonly\n\t\t(focus)=\"picker.open()\"\n\t\t[matDatepicker]=\"picker\"\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[required]=\"required\"\n\t\t[matDatepickerFilter]=\"filterFn\"\n\t/>\n\t<mat-datepicker-toggle matSuffix [for]=\"picker\"></mat-datepicker-toggle>\n\t<mat-datepicker\n\t\t#picker\n\t\tdisabled=\"false\"\n\t\tstartView=\"multi-year\"\n\t\t(yearSelected)=\"_chosenYearHandler($event)\"\n\t\t(monthSelected)=\"_chosenMonthHandler($event, picker)\"\n\t></mat-datepicker>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                providers: [
                    // `MomentDateAdapter` and `MAT_MOMENT_DATE_FORMATS` can be automatically provided by importing
                    // `MatMomentDateModule` in your applications root module. We provide it at the component level
                    // here, due to limitations of our example generation script.
                    { provide: MAT_DATE_LOCALE, useValue: "en-AU" },
                    {
                        provide: DateAdapter,
                        useClass: MomentDateAdapter,
                        deps: [MAT_DATE_LOCALE]
                    },
                    { provide: MAT_DATE_FORMATS, useValue: MONTH_YEAR_FORMAT }
                ],
                styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
            }] }
];
/** @nocollapse */
ExpiryMonthFormFieldComponent.ctorParameters = () => [
    { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
];
ExpiryMonthFormFieldComponent.propDecorators = {
    filterFn: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SelectFormFieldComponent extends FromFieldControl {
    /**
     * @param {?} __formGroupDirective
     */
    constructor(__formGroupDirective) {
        super(__formGroupDirective);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.optionDisplayFn = this.optionDisplayFn || this.__displayFn;
        this.optionClassFn = this.optionClassFn || this.__classFn;
    }
    /**
     * @private
     * @param {?=} option
     * @return {?}
     */
    __classFn(option) {
        return undefined;
    }
    /**
     * @private
     * @param {?=} option
     * @return {?}
     */
    __displayFn(option) {
        if (!option) {
            return undefined;
        }
        return this.__isPrimitiveType(option)
            ? option
            : option[this.displayProperty];
    }
    /**
     * @private
     * @param {?} val
     * @return {?}
     */
    __isPrimitiveType(val) {
        /** @type {?} */
        const primitiveTypes = [
            "boolean",
            "number",
            "string",
            "undefined",
            "null"
        ];
        /** @type {?} */
        const valType = typeof val;
        return primitiveTypes.indexOf(valType) >= 0;
    }
}
SelectFormFieldComponent.decorators = [
    { type: Component, args: [{
                selector: "tc-select-form-field",
                template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<mat-select [formControl]=\"_control\" [required]=\"required\">\n\t\t<ng-container *ngIf=\"options; else observableOptions\">\n\t\t\t<mat-option\n\t\t\t\t*ngFor=\"let option of options\"\n\t\t\t\t[value]=\"option\"\n\t\t\t\t[ngClass]=\"optionClassFn(option)\"\n\t\t\t>\n\t\t\t\t{{ optionDisplayFn(option) }}\n\t\t\t</mat-option>\n\t\t</ng-container>\n\t\t<ng-template #observableOptions>\n\t\t\t<mat-option\n\t\t\t\t*ngFor=\"let option of options$ | async\"\n\t\t\t\t[value]=\"option\"\n\t\t\t\t[ngClass]=\"optionClassFn(option)\"\n\t\t\t>\n\t\t\t\t{{ optionDisplayFn(option) }}\n\t\t\t</mat-option>\n\t\t</ng-template>\n\t</mat-select>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host mat-spinner{position:absolute;right:0;bottom:5px}:host img.valid-tick{position:absolute;right:0;top:66%;margin-top:-6px}"]
            }] }
];
/** @nocollapse */
SelectFormFieldComponent.ctorParameters = () => [
    { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
];
SelectFormFieldComponent.propDecorators = {
    options: [{ type: Input }],
    options$: [{ type: Input }],
    displayProperty: [{ type: Input }],
    classList: [{ type: Input }],
    optionDisplayFn: [{ type: Input }],
    optionClassFn: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TryCatchMaterialFormsModule {
}
TryCatchMaterialFormsModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule,
                    ReactiveFormsModule,
                    MatCommonModule,
                    MatInputModule,
                    MatDatepickerModule,
                    MatFormFieldModule,
                    MatMomentDateModule,
                    MatProgressSpinnerModule,
                    MatSelectModule,
                    TextMaskModule,
                    TryCatchMaterialExtensionModule
                ],
                declarations: [
                    InputFormFieldComponent,
                    AutoCompleteFormFieldComponent,
                    NumberFormFieldComponent,
                    TextAreaFormFieldComponent,
                    MaskedFormFieldComponent,
                    MomentFormFieldComponent,
                    ExpiryMonthFormFieldComponent,
                    SelectFormFieldComponent
                ],
                exports: [
                    InputFormFieldComponent,
                    AutoCompleteFormFieldComponent,
                    NumberFormFieldComponent,
                    TextAreaFormFieldComponent,
                    MaskedFormFieldComponent,
                    MomentFormFieldComponent,
                    ExpiryMonthFormFieldComponent,
                    SelectFormFieldComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { TryCatchMaterialFormsModule, InputFormFieldComponent, AutoCompleteFormFieldComponent, NumberFormFieldComponent, TextAreaFormFieldComponent, MaskedFormFieldComponent, DATE_FORMATS, MomentFormFieldComponent, MONTH_YEAR_FORMAT, ExpiryMonthFormFieldComponent, SelectFormFieldComponent, FormFieldControlType, FormFieldEventType, FromFieldControl as ɵa };

//# sourceMappingURL=trycatch-material-forms.js.map