import { CommonModule } from '@angular/common';
import { TextMaskModule } from 'angular2-text-mask';
import { TryCatchMaterialExtensionModule } from '@trycatch/material-extension';
import '@angular/cdk/text-field';
import { take } from 'rxjs/operators';
import createAutoCorrectedDatePipe from 'text-mask-addons/dist/createAutoCorrectedDatePipe';
import { MAT_DATE_LOCALE, DateAdapter, MAT_DATE_FORMATS, MatCommonModule, MatFormFieldModule, MatInputModule, MatDatepickerModule, MatSelectModule, MatProgressSpinnerModule } from '@angular/material';
import { MomentDateAdapter, MatMomentDateModule } from '@angular/material-moment-adapter';
import * as moment_ from 'moment';
import { __extends } from 'tslib';
import { Input, Output, EventEmitter, Component, Optional, Inject, NgZone, ViewChild, NgModule } from '@angular/core';
import { FormGroupDirective, FormControl, FormsModule, ReactiveFormsModule } from '@angular/forms';
import 'rxjs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
var FormFieldControlType = {
    INPUT_TEXT: "InputText",
    INPUT_NUMBER: "InputNumber",
    AUTOCOMPLETE: "AutoComplete",
};
/** @enum {string} */
var FormFieldEventType = {
    INPUT_CHANGED: "input_changed",
    OPTION_SELECTED: "option_selected",
    CONTROL_CREATED: "control_created",
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} event
 * @return {?}
 */
function numericKeysOnly(event) {
    /** @type {?} */
    var charCode = event.which ? event.which : event.keyCode;
    if (charCode >= 48 && charCode <= 57) {
        return true;
    }
    return false;
}
/**
 * @param {?} event
 * @return {?}
 */
function numericKeysOnlyWithDecimalPoint(event) {
    /** @type {?} */
    var charCode = event.which ? event.which : event.keyCode;
    if ((charCode >= 48 && charCode <= 57) || charCode === 46) {
        return true;
    }
    return false;
}
/**
 * @param {?} event
 * @return {?}
 */
function alphaNumericKeysOnly(event) {
    /** @type {?} */
    var charCode = event.which ? event.which : event.keyCode;
    if (charCode === 32 ||
        (charCode >= 97 && charCode <= 122) ||
        (charCode >= 65 && charCode <= 90) ||
        (charCode >= 48 && charCode <= 57) // numeric keys
    ) {
        return true;
    }
    return false;
}
/**
 * @param {?} event
 * @return {?}
 */
function alphaKeysOnly(event) {
    /** @type {?} */
    var charCode = event.which ? event.which : event.keyCode;
    if (charCode >= 48 && charCode <= 57) {
        // numeric keys
        return false;
    }
    if (charCode >= 33 && charCode <= 38) {
        // special keys
        return false;
    }
    if (charCode >= 40 && charCode <= 44) {
        // special keys
        return false;
    }
    if (charCode >= 46 && charCode <= 47) {
        // special keys
        return false;
    }
    if (charCode >= 58 && charCode <= 64) {
        // special keys
        return false;
    }
    if (charCode >= 91 && charCode <= 95) {
        // special keys
        return false;
    }
    if (charCode >= 123 && charCode <= 126) {
        // special keys
        return false;
    }
    if (charCode >= 161 && charCode <= 191) {
        // special keys
        return false;
    }
    if (charCode === 8217) {
        // special keys
        return false;
    }
    return true;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
var FromFieldControl = /** @class */ (function () {
    function FromFieldControl(__formGroupDirective) {
        this.keyRestrictionsFunctionMap = {
            "numeric-only": numericKeysOnly,
            "numeric-with-decimal-point": numericKeysOnlyWithDecimalPoint,
            "alpha-numeric": alphaNumericKeysOnly,
            "alpha-only": alphaKeysOnly
        };
        this.event = new EventEmitter();
        this._keyRestrictionFunction = this.__keyRestrictionFunction;
        this.formGroupDirective = __formGroupDirective;
        this.event.emit((/** @type {?} */ ({
            type: FormFieldEventType.CONTROL_CREATED
        })));
        if (!__formGroupDirective) {
            console.error("form field controls can be used within form group container");
        }
    }
    Object.defineProperty(FromFieldControl.prototype, "keyRestrictions", {
        set: /**
         * @param {?} val
         * @return {?}
         */
        function (val) {
            if (!!val) {
                this._keyRestrictionFunction = this.keyRestrictionsFunctionMap[val];
            }
            else {
                this._keyRestrictionFunction = this.__keyRestrictionFunction;
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    FromFieldControl.prototype.__keyRestrictionFunction = /**
     * @private
     * @param {?} event
     * @return {?}
     */
    function (event) {
        return true;
    };
    /**
     * @param {?} changes
     * @return {?}
     */
    FromFieldControl.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if ("controlName" in changes) {
            if (this.formGroupDirective) {
                this.formGroup = this.formGroupDirective.form;
                this._control = this.formGroup.controls[changes.controlName.currentValue];
                this.__setValidators();
                this.__pendingSubscription();
            }
        }
    };
    /**
     * @return {?}
     */
    FromFieldControl.prototype._runReplacement = /**
     * @return {?}
     */
    function () {
        if (this.replacementMap &&
            this.replacementMap.length > 0 &&
            this._control) {
            /** @type {?} */
            var val_1 = this._control.value.toString();
            if (val_1) {
                this.replacementMap.forEach((/**
                 * @param {?} rm
                 * @return {?}
                 */
                function (rm) {
                    /** @type {?} */
                    var regEx = new RegExp(rm[0].source, rm[0].flags + "g");
                    val_1 = val_1.replace(regEx, rm[1]);
                }));
                this._control.setValue(val_1);
            }
        }
    };
    /**
     * @private
     * @return {?}
     */
    FromFieldControl.prototype.__setValidators = /**
     * @private
     * @return {?}
     */
    function () {
        if (this.validationFn) {
            this._control.setValidators(this.validationFn);
        }
    };
    /**
     * @private
     * @return {?}
     */
    FromFieldControl.prototype.__pendingSubscription = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        if (this._control) {
            this._control.statusChanges.subscribe((/**
             * @param {?} status
             * @return {?}
             */
            function (status) {
                /** @type {?} */
                var msg = _this.controlName + " status is : " + status;
                console.log(msg);
                console.log(_this._control.errors);
                _this._status = status;
            }));
        }
    };
    Object.defineProperty(FromFieldControl.prototype, "__control", {
        get: /**
         * @protected
         * @return {?}
         */
        function () {
            return this._control;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} controlType
     * @return {?}
     */
    FromFieldControl.prototype.__controlInfo = /**
     * @param {?} controlType
     * @return {?}
     */
    function (controlType) {
        return {
            name: this.controlName,
            type: controlType,
            abstractControl: this.__control
        };
    };
    Object.defineProperty(FromFieldControl.prototype, "valid", {
        get: /**
         * @return {?}
         */
        function () {
            return this.__control
                ? this.__control.dirty &&
                    !this.__control.errors &&
                    !this.__control.pending
                : undefined;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FromFieldControl.prototype, "error", {
        get: /**
         * @return {?}
         */
        function () {
            if (this.__control) {
                /** @type {?} */
                var ctrl_1 = this.__control;
                if (ctrl_1.errors) {
                    if (ctrl_1.hasError("required")) {
                        return this.label + " is required";
                    }
                    if (ctrl_1.hasError("error")) {
                        return "" + ctrl_1.errors["error"];
                    }
                    if (this.errorFn) {
                        /** @type {?} */
                        var err = this.errorFn(ctrl_1.errors, this.parent);
                        if (err) {
                            return err;
                        }
                        else {
                            setTimeout((/**
                             * @return {?}
                             */
                            function () {
                                ctrl_1.setErrors(null);
                            }));
                        }
                    }
                }
            }
            return undefined;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @protected
     * @param {?} val
     * @return {?}
     */
    FromFieldControl.prototype._cleanMaskValue = /**
     * @protected
     * @param {?} val
     * @return {?}
     */
    function (val) {
        return val ? val.replace(/[^\x00-\x7F]/g, "") : val;
    };
    FromFieldControl.propDecorators = {
        label: [{ type: Input }],
        placeholder: [{ type: Input }],
        hint: [{ type: Input }],
        controlName: [{ type: Input }],
        required: [{ type: Input }],
        validImage: [{ type: Input }],
        appearance: [{ type: Input }],
        floatLabel: [{ type: Input }],
        color: [{ type: Input }],
        invalidImage: [{ type: Input }],
        parent: [{ type: Input }],
        validationFn: [{ type: Input }],
        errorFn: [{ type: Input }],
        keyRestrictions: [{ type: Input }],
        replacementMap: [{ type: Input }],
        event: [{ type: Output }]
    };
    return FromFieldControl;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var InputFormFieldComponent = /** @class */ (function (_super) {
    __extends(InputFormFieldComponent, _super);
    function InputFormFieldComponent(__formGroupDirective) {
        var _this = _super.call(this, __formGroupDirective) || this;
        _this.maxLength = -1;
        _this.minLength = -1;
        return _this;
    }
    /**
     * @return {?}
     */
    InputFormFieldComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    InputFormFieldComponent.decorators = [
        { type: Component, args: [{
                    selector: "tc-input-form-field",
                    template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<input\n\t\tmatInput\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t(keypress)=\"_keyRestrictionFunction($event)\"\n\t\t(blur)=\"_runReplacement()\"\n\t\t[required]=\"required\"\n\t\t[maxlength]=\"maxLength > 0 ? maxLength : undefined\"\n\t\ttype=\"text\"\n\t/>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<mat-spinner\n\t\t*ngIf=\"_status === 'PENDING'\"\n\t\t[diameter]=\"18\"\n\t\t[strokeWidth]=\"3\"\n\t></mat-spinner>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                    styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
                }] }
    ];
    /** @nocollapse */
    InputFormFieldComponent.ctorParameters = function () { return [
        { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
    ]; };
    InputFormFieldComponent.propDecorators = {
        maxLength: [{ type: Input }],
        minLength: [{ type: Input }]
    };
    return InputFormFieldComponent;
}(FromFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AutoCompleteFormFieldComponent = /** @class */ (function (_super) {
    __extends(AutoCompleteFormFieldComponent, _super);
    function AutoCompleteFormFieldComponent(__formGroupDirective) {
        var _this = _super.call(this, __formGroupDirective) || this;
        _this.charThreshold = 2;
        return _this;
    }
    /**
     * @return {?}
     */
    AutoCompleteFormFieldComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    /**
     * @param {?} eventData
     * @return {?}
     */
    AutoCompleteFormFieldComponent.prototype._optionSelected = /**
     * @param {?} eventData
     * @return {?}
     */
    function (eventData) {
        /** @type {?} */
        var eventVal = (/** @type {?} */ ({
            type: FormFieldEventType.OPTION_SELECTED,
            control: this.__controlInfo(FormFieldControlType.AUTOCOMPLETE)
        }));
        if (eventData) {
            eventVal.data = eventData;
        }
        else {
            this.__control.reset();
        }
        this.event.emit(eventVal);
    };
    /**
     * @param {?} eventData
     * @return {?}
     */
    AutoCompleteFormFieldComponent.prototype._inputChanged = /**
     * @param {?} eventData
     * @return {?}
     */
    function (eventData) {
        this.event.emit((/** @type {?} */ ({
            type: FormFieldEventType.INPUT_CHANGED,
            data: eventData,
            control: this.__controlInfo(FormFieldControlType.AUTOCOMPLETE)
        })));
    };
    AutoCompleteFormFieldComponent.decorators = [
        { type: Component, args: [{
                    selector: "tc-auto-complete-form-field",
                    template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<trycatch-mat-auto-complete\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[options]=\"options\"\n\t\t[displayProperty]=\"displayProperty\"\n\t\t[displayFn]=\"displayFn\"\n\t\t[charThreshold]=\"charThreshold\"\n\t\t[optionDisplayFn]=\"optionDisplayFn\"\n\t\t[optionClassFn]=\"optionClassFn\"\n\t\t[required]=\"required\"\n\t\t[options$]=\"options$\"\n\t\t(optionSelected)=\"_optionSelected($event)\"\n\t\t(inputChanged)=\"_inputChanged($event)\"\n\t\t[classList]=\"classList\"\n\t>\n\t</trycatch-mat-auto-complete>\n\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                    styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
                }] }
    ];
    /** @nocollapse */
    AutoCompleteFormFieldComponent.ctorParameters = function () { return [
        { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
    ]; };
    AutoCompleteFormFieldComponent.propDecorators = {
        options: [{ type: Input }],
        options$: [{ type: Input }],
        displayProperty: [{ type: Input }],
        displayFn: [{ type: Input }],
        optionDisplayFn: [{ type: Input }],
        optionClassFn: [{ type: Input }],
        charThreshold: [{ type: Input }],
        classList: [{ type: Input }]
    };
    return AutoCompleteFormFieldComponent;
}(FromFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NumberFormFieldComponent = /** @class */ (function (_super) {
    __extends(NumberFormFieldComponent, _super);
    function NumberFormFieldComponent(__formGroupDirective) {
        var _this = _super.call(this, __formGroupDirective) || this;
        _this.step = 0.1;
        return _this;
    }
    /**
     * @return {?}
     */
    NumberFormFieldComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this._control.valueChanges.subscribe((/**
         * @param {?} val
         * @return {?}
         */
        function (val) {
            if (val) {
                if (_this.minValue) {
                    if (val < _this.minValue) {
                        console.log("setting up min value error");
                        _this._control.setErrors({
                            error: "value out of range. Value less then min allowed value"
                        });
                    }
                }
                if (_this.maxValue) {
                    if (val > _this.maxValue) {
                        console.log("setting up max value error");
                        _this._control.setErrors({
                            error: "value out of range. Value greater then max allowed value"
                        });
                    }
                }
            }
            else {
                _this._control.setErrors(null);
            }
        }));
    };
    NumberFormFieldComponent.decorators = [
        { type: Component, args: [{
                    selector: "tc-number-form-field",
                    template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<input\n\t\tmatInput\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[required]=\"required\"\n\t\ttype=\"number\"\n\t\t[step]=\"step\"\n\t\t[attr.min]=\"minValue\"\n\t\t[attr.max]=\"maxValue\"\n\t/>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                    styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
                }] }
    ];
    /** @nocollapse */
    NumberFormFieldComponent.ctorParameters = function () { return [
        { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
    ]; };
    NumberFormFieldComponent.propDecorators = {
        step: [{ type: Input }],
        minValue: [{ type: Input }],
        maxValue: [{ type: Input }]
    };
    return NumberFormFieldComponent;
}(FromFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TextAreaFormFieldComponent = /** @class */ (function (_super) {
    __extends(TextAreaFormFieldComponent, _super);
    function TextAreaFormFieldComponent(ngZone, __formGroupDirective) {
        var _this = _super.call(this, __formGroupDirective) || this;
        _this.ngZone = ngZone;
        _this.minRows = 3;
        _this.maxRows = 15;
        return _this;
    }
    /**
     * @return {?}
     */
    TextAreaFormFieldComponent.prototype.triggerResize = /**
     * @return {?}
     */
    function () {
        var _this = this;
        // Wait for changes to be applied, then trigger textarea resize.
        this.ngZone.onStable
            .pipe(take(1))
            .subscribe((/**
         * @return {?}
         */
        function () { return _this.autosize.resizeToFitContent(true); }));
    };
    /**
     * @return {?}
     */
    TextAreaFormFieldComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    TextAreaFormFieldComponent.decorators = [
        { type: Component, args: [{
                    selector: "tc-text-area-form-field",
                    template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<textarea\n\t\tmatInput\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[required]=\"required\"\n\t\ttype=\"text\"\n\t\tcdkTextareaAutosize\n\t\t#autosize=\"cdkTextareaAutosize\"\n\t\t[cdkAutosizeMinRows]=\"minRows\"\n\t\t[cdkAutosizeMaxRows]=\"maxRows\"\n\t></textarea>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                    styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
                }] }
    ];
    /** @nocollapse */
    TextAreaFormFieldComponent.ctorParameters = function () { return [
        { type: NgZone },
        { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
    ]; };
    TextAreaFormFieldComponent.propDecorators = {
        minRows: [{ type: Input }],
        maxRows: [{ type: Input }],
        cols: [{ type: Input }],
        autosize: [{ type: ViewChild, args: ["autosize",] }]
    };
    return TextAreaFormFieldComponent;
}(FromFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var MaskedFormFieldComponent = /** @class */ (function (_super) {
    __extends(MaskedFormFieldComponent, _super);
    function MaskedFormFieldComponent(__formGroupDirective) {
        var _this = _super.call(this, __formGroupDirective) || this;
        _this._placeholderDate = "01";
        _this._placeholderMonth = "01";
        _this._placeholderYear = "1900";
        _this.maxLength = -1;
        _this.minLength = -1;
        return _this;
    }
    Object.defineProperty(MaskedFormFieldComponent.prototype, "mask", {
        set: /**
         * @param {?} val
         * @return {?}
         */
        function (val) {
            if (val) {
                this._mask = {
                    mask: val,
                    guide: true,
                    placeholderChar: "\u2000"
                };
                if (this._autoCorrectedDatePipe) {
                    this._mask.pipe = this._autoCorrectedDatePipe;
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MaskedFormFieldComponent.prototype, "dateFormat", {
        set: /**
         * @param {?} format
         * @return {?}
         */
        function (format) {
            if (format) {
                this._autoCorrectedDatePipe = createAutoCorrectedDatePipe(format);
                this.placeholder = format
                    .replace("dd", this._placeholderDate)
                    .replace("mm", this._placeholderMonth)
                    .replace("yyyy", this._placeholderYear);
                if (this._mask) {
                    this._mask.pipe = this._autoCorrectedDatePipe;
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    MaskedFormFieldComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    MaskedFormFieldComponent.decorators = [
        { type: Component, args: [{
                    selector: "tc-masked-form-field",
                    template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<input\n\t\tmatInput\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[required]=\"required\"\n\t\t[maxlength]=\"maxLength > 0 ? maxLength : undefined\"\n\t\t(keypress)=\"_keyRestrictionFunction($event)\"\n\t\ttype=\"text\"\n\t\t[textMask]=\"_mask\"\n\t/>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<mat-spinner\n\t\t*ngIf=\"_status === 'PENDING'\"\n\t\t[diameter]=\"18\"\n\t\t[strokeWidth]=\"3\"\n\t></mat-spinner>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                    styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
                }] }
    ];
    /** @nocollapse */
    MaskedFormFieldComponent.ctorParameters = function () { return [
        { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
    ]; };
    MaskedFormFieldComponent.propDecorators = {
        maxLength: [{ type: Input }],
        minLength: [{ type: Input }],
        mask: [{ type: Input }],
        dateFormat: [{ type: Input }]
    };
    return MaskedFormFieldComponent;
}(FromFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var DATE_FORMATS = {
    parse: {
        dateInput: "L"
    },
    display: {
        dateInput: "L",
        monthYearLabel: "MMM YYYY",
        dateA11yLabel: "L",
        monthYearA11yLabel: "MMMM YYYY"
    }
};
/** @type {?} */
var moment = moment_;
var MomentFormFieldComponent = /** @class */ (function (_super) {
    __extends(MomentFormFieldComponent, _super);
    function MomentFormFieldComponent(__formGroupDirective) {
        var _this = _super.call(this, __formGroupDirective) || this;
        _this.__dateMask = [
            /\d/,
            /\d/,
            "/",
            /\d/,
            /\d/,
            "/",
            /[1-2]/,
            /\d/,
            /\d/,
            /\d/
        ];
        _this.__dpControl = new FormControl();
        _this._mask = {
            mask: _this.__dateMask,
            guide: true,
            placeholderChar: "\u2000",
            pipe: createAutoCorrectedDatePipe("dd/mm/yyyy")
        };
        _this.placeholder = "01/01/1900";
        _this.hint = "format: 01/01/1900";
        return _this;
    }
    Object.defineProperty(MomentFormFieldComponent.prototype, "startDate", {
        set: /**
         * @param {?} val
         * @return {?}
         */
        function (val) {
            /** @type {?} */
            var mmt = moment(val, "DD/MM/YYYY");
            if (mmt.isValid()) {
                // this.__dpControl.setValue(mmt, { emitEvent: false });
                this._startDate = mmt;
                this._bkpStartDate = mmt;
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    MomentFormFieldComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.__dpControl.valueChanges.subscribe((/**
         * @param {?} val
         * @return {?}
         */
        function (val) {
            /** @type {?} */
            var dt = val.format("DD/MM/YYYY");
            _this._control.setValue(dt, { emitEvent: false });
            _this._control.markAsTouched();
            _this._startDate = val;
        }));
        this._control.valueChanges.subscribe((/**
         * @param {?} val
         * @return {?}
         */
        function (val) {
            /** @type {?} */
            var dt = _this._cleanMaskValue(val);
            if (dt.length === 10) {
                /** @type {?} */
                var mmt = moment(dt, "DD/MM/YYYY");
                if (mmt.isValid()) {
                    _this.__dpControl.setValue(mmt, { emitEvent: false });
                    _this._startDate = mmt;
                }
            }
            else if (dt.length === 0) {
                _this._startDate = _this._bkpStartDate;
            }
        }));
    };
    MomentFormFieldComponent.decorators = [
        { type: Component, args: [{
                    selector: "tc-moment-form-field",
                    template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<input\n\t\tmatInput\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[required]=\"required\"\n\t\t[textMask]=\"_mask\"\n\t/>\n\t<input\n\t\tmatInput\n\t\t[hidden]=\"true\"\n\t\t[formControl]=\"__dpControl\"\n\t\t[matDatepicker]=\"picker\"\n\t\t[required]=\"required\"\n\t\t[matDatepickerFilter]=\"dateFilterFn\"\n\t/>\n\t<mat-datepicker-toggle matSuffix [for]=\"picker\"></mat-datepicker-toggle>\n\t<mat-datepicker\n\t\t#picker\n\t\tdisabled=\"false\"\n\t\tstartView=\"month\"\n\t\t[startAt]=\"_startDate\"\n\t></mat-datepicker>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                    providers: [
                        // `MomentDateAdapter` and `MAT_MOMENT_DATE_FORMATS` can be automatically provided by importing
                        // `MatMomentDateModule` in your applications root module. We provide it at the component level
                        // here, due to limitations of our example generation script.
                        { provide: MAT_DATE_LOCALE, useValue: "en-AU" },
                        {
                            provide: DateAdapter,
                            useClass: MomentDateAdapter,
                            deps: [MAT_DATE_LOCALE]
                        },
                        { provide: MAT_DATE_FORMATS, useValue: DATE_FORMATS }
                    ],
                    styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}:host mat-datepicker-toggle ::ng-deep button{width:30px;height:30px}:host mat-datepicker-toggle ::ng-deep button span.mat-button-wrapper{display:block}"]
                }] }
    ];
    /** @nocollapse */
    MomentFormFieldComponent.ctorParameters = function () { return [
        { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
    ]; };
    MomentFormFieldComponent.propDecorators = {
        dateFilterFn: [{ type: Input }],
        startDate: [{ type: Input }]
    };
    return MomentFormFieldComponent;
}(FromFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var MONTH_YEAR_FORMAT = {
    parse: {
        dateInput: "MM/YYYY"
    },
    display: {
        dateInput: "MM/YYYY",
        monthYearLabel: "MMM YYYY",
        dateA11yLabel: "LL",
        monthYearA11yLabel: "MMMM YYYY"
    }
};
/** @type {?} */
var moment$1 = moment_;
var ExpiryMonthFormFieldComponent = /** @class */ (function (_super) {
    __extends(ExpiryMonthFormFieldComponent, _super);
    function ExpiryMonthFormFieldComponent(__formGroupDirective) {
        var _this = _super.call(this, __formGroupDirective) || this;
        _this.__val = moment$1();
        return _this;
    }
    /**
     * @return {?}
     */
    ExpiryMonthFormFieldComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (this._control.value) {
            this.__val = moment$1(this._control.value, "MM/YYYY");
            // } else {
            // 	this._control.setValue(this.__val);
        }
    };
    /**
     * @param {?} normalizedYear
     * @return {?}
     */
    ExpiryMonthFormFieldComponent.prototype._chosenYearHandler = /**
     * @param {?} normalizedYear
     * @return {?}
     */
    function (normalizedYear) {
        console.log("control : ", this._control);
        /** @type {?} */
        var ctrlValue = this._control.value || this.__val;
        console.log("control value : ", ctrlValue);
        ctrlValue.year(normalizedYear.year());
        if (this._control.value) {
            this._control.setValue(ctrlValue);
        }
        else {
            this.__val = ctrlValue;
        }
        console.log("control value : ", this._control.value);
    };
    /**
     * @param {?} normlizedMonth
     * @param {?} datepicker
     * @return {?}
     */
    ExpiryMonthFormFieldComponent.prototype._chosenMonthHandler = /**
     * @param {?} normlizedMonth
     * @param {?} datepicker
     * @return {?}
     */
    function (normlizedMonth, datepicker) {
        /** @type {?} */
        var ctrlValue = this._control.value || this.__val;
        ctrlValue.month(normlizedMonth.month());
        if (this._control.value) {
            this._control.setValue(ctrlValue);
        }
        else {
            this.__val = ctrlValue;
            this._control.setValue(this.__val);
        }
        datepicker.close();
    };
    ExpiryMonthFormFieldComponent.decorators = [
        { type: Component, args: [{
                    selector: "tc-expiry-month-form-field",
                    template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<input\n\t\tmatInput\n\t\treadonly\n\t\t(focus)=\"picker.open()\"\n\t\t[matDatepicker]=\"picker\"\n\t\t[formControl]=\"_control\"\n\t\t[placeholder]=\"placeholder\"\n\t\t[required]=\"required\"\n\t\t[matDatepickerFilter]=\"filterFn\"\n\t/>\n\t<mat-datepicker-toggle matSuffix [for]=\"picker\"></mat-datepicker-toggle>\n\t<mat-datepicker\n\t\t#picker\n\t\tdisabled=\"false\"\n\t\tstartView=\"multi-year\"\n\t\t(yearSelected)=\"_chosenYearHandler($event)\"\n\t\t(monthSelected)=\"_chosenMonthHandler($event, picker)\"\n\t></mat-datepicker>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                    providers: [
                        // `MomentDateAdapter` and `MAT_MOMENT_DATE_FORMATS` can be automatically provided by importing
                        // `MatMomentDateModule` in your applications root module. We provide it at the component level
                        // here, due to limitations of our example generation script.
                        { provide: MAT_DATE_LOCALE, useValue: "en-AU" },
                        {
                            provide: DateAdapter,
                            useClass: MomentDateAdapter,
                            deps: [MAT_DATE_LOCALE]
                        },
                        { provide: MAT_DATE_FORMATS, useValue: MONTH_YEAR_FORMAT }
                    ],
                    styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host img.valid-tick{position:absolute;right:0;top:50%;margin-top:-6px}:host mat-spinner{position:absolute;right:0;bottom:5px}"]
                }] }
    ];
    /** @nocollapse */
    ExpiryMonthFormFieldComponent.ctorParameters = function () { return [
        { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
    ]; };
    ExpiryMonthFormFieldComponent.propDecorators = {
        filterFn: [{ type: Input }]
    };
    return ExpiryMonthFormFieldComponent;
}(FromFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var SelectFormFieldComponent = /** @class */ (function (_super) {
    __extends(SelectFormFieldComponent, _super);
    function SelectFormFieldComponent(__formGroupDirective) {
        return _super.call(this, __formGroupDirective) || this;
    }
    /**
     * @return {?}
     */
    SelectFormFieldComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.optionDisplayFn = this.optionDisplayFn || this.__displayFn;
        this.optionClassFn = this.optionClassFn || this.__classFn;
    };
    /**
     * @private
     * @param {?=} option
     * @return {?}
     */
    SelectFormFieldComponent.prototype.__classFn = /**
     * @private
     * @param {?=} option
     * @return {?}
     */
    function (option) {
        return undefined;
    };
    /**
     * @private
     * @param {?=} option
     * @return {?}
     */
    SelectFormFieldComponent.prototype.__displayFn = /**
     * @private
     * @param {?=} option
     * @return {?}
     */
    function (option) {
        if (!option) {
            return undefined;
        }
        return this.__isPrimitiveType(option)
            ? option
            : option[this.displayProperty];
    };
    /**
     * @private
     * @param {?} val
     * @return {?}
     */
    SelectFormFieldComponent.prototype.__isPrimitiveType = /**
     * @private
     * @param {?} val
     * @return {?}
     */
    function (val) {
        /** @type {?} */
        var primitiveTypes = [
            "boolean",
            "number",
            "string",
            "undefined",
            "null"
        ];
        /** @type {?} */
        var valType = typeof val;
        return primitiveTypes.indexOf(valType) >= 0;
    };
    SelectFormFieldComponent.decorators = [
        { type: Component, args: [{
                    selector: "tc-select-form-field",
                    template: "<mat-form-field\n\t[color]=\"color\"\n\t[appearance]=\"appearance\"\n\t[floatLabel]=\"floatLabel\"\n>\n\t<mat-label [innerHtml]=\"label\"></mat-label>\n\t<mat-select [formControl]=\"_control\" [required]=\"required\">\n\t\t<ng-container *ngIf=\"options; else observableOptions\">\n\t\t\t<mat-option\n\t\t\t\t*ngFor=\"let option of options\"\n\t\t\t\t[value]=\"option\"\n\t\t\t\t[ngClass]=\"optionClassFn(option)\"\n\t\t\t>\n\t\t\t\t{{ optionDisplayFn(option) }}\n\t\t\t</mat-option>\n\t\t</ng-container>\n\t\t<ng-template #observableOptions>\n\t\t\t<mat-option\n\t\t\t\t*ngFor=\"let option of options$ | async\"\n\t\t\t\t[value]=\"option\"\n\t\t\t\t[ngClass]=\"optionClassFn(option)\"\n\t\t\t>\n\t\t\t\t{{ optionDisplayFn(option) }}\n\t\t\t</mat-option>\n\t\t</ng-template>\n\t</mat-select>\n\t<mat-hint [innerHtml]=\"hint\"></mat-hint>\n\t<mat-error [innerHtml]=\"error\"></mat-error>\n\t<img *ngIf=\"valid && validImage\" class=\"valid-tick\" [src]=\"validImage\" />\n</mat-form-field>\n",
                    styles: [":host{width:100%;display:block}:host mat-form-field{width:100%;line-height:inherit}:host mat-spinner{position:absolute;right:0;bottom:5px}:host img.valid-tick{position:absolute;right:0;top:66%;margin-top:-6px}"]
                }] }
    ];
    /** @nocollapse */
    SelectFormFieldComponent.ctorParameters = function () { return [
        { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Inject, args: [FormGroupDirective,] }] }
    ]; };
    SelectFormFieldComponent.propDecorators = {
        options: [{ type: Input }],
        options$: [{ type: Input }],
        displayProperty: [{ type: Input }],
        classList: [{ type: Input }],
        optionDisplayFn: [{ type: Input }],
        optionClassFn: [{ type: Input }]
    };
    return SelectFormFieldComponent;
}(FromFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TryCatchMaterialFormsModule = /** @class */ (function () {
    function TryCatchMaterialFormsModule() {
    }
    TryCatchMaterialFormsModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule,
                        FormsModule,
                        ReactiveFormsModule,
                        MatCommonModule,
                        MatInputModule,
                        MatDatepickerModule,
                        MatFormFieldModule,
                        MatMomentDateModule,
                        MatProgressSpinnerModule,
                        MatSelectModule,
                        TextMaskModule,
                        TryCatchMaterialExtensionModule
                    ],
                    declarations: [
                        InputFormFieldComponent,
                        AutoCompleteFormFieldComponent,
                        NumberFormFieldComponent,
                        TextAreaFormFieldComponent,
                        MaskedFormFieldComponent,
                        MomentFormFieldComponent,
                        ExpiryMonthFormFieldComponent,
                        SelectFormFieldComponent
                    ],
                    exports: [
                        InputFormFieldComponent,
                        AutoCompleteFormFieldComponent,
                        NumberFormFieldComponent,
                        TextAreaFormFieldComponent,
                        MaskedFormFieldComponent,
                        MomentFormFieldComponent,
                        ExpiryMonthFormFieldComponent,
                        SelectFormFieldComponent
                    ]
                },] }
    ];
    return TryCatchMaterialFormsModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { TryCatchMaterialFormsModule, InputFormFieldComponent, AutoCompleteFormFieldComponent, NumberFormFieldComponent, TextAreaFormFieldComponent, MaskedFormFieldComponent, DATE_FORMATS, MomentFormFieldComponent, MONTH_YEAR_FORMAT, ExpiryMonthFormFieldComponent, SelectFormFieldComponent, FormFieldControlType, FormFieldEventType, FromFieldControl as ɵa };

//# sourceMappingURL=trycatch-material-forms.js.map